// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PokerWinning.hpp"

using namespace Dawn;

float_t PokerWinning::getWinningTypeConfidence(enum PokerWinningType type) {
  switch(type) {
    case POKER_WINNING_TYPE_ROYAL_FLUSH:
      return POKER_WINNING_CONFIDENCE_ROYAL_FLUSH;
    case POKER_WINNING_TYPE_STRAIGHT_FLUSH:
      return POKER_WINNING_CONFIDENCE_STRAIGHT_FLUSH;
    case POKER_WINNING_TYPE_FOUR_OF_A_KIND:
      return POKER_WINNING_CONFIDENCE_FOUR_OF_A_KIND;
    case POKER_WINNING_TYPE_FULL_HOUSE:
      return POKER_WINNING_CONFIDENCE_FULL_HOUSE;
    case POKER_WINNING_TYPE_FLUSH:
      return POKER_WINNING_CONFIDENCE_FLUSH;
    case POKER_WINNING_TYPE_STRAIGHT:
      return POKER_WINNING_CONFIDENCE_STRAIGHT;
    case POKER_WINNING_TYPE_THREE_OF_A_KIND:
      return POKER_WINNING_CONFIDENCE_THREE_OF_A_KIND;
    case POKER_WINNING_TYPE_TWO_PAIR:
      return POKER_WINNING_CONFIDENCE_TWO_PAIR;
    case POKER_WINNING_TYPE_PAIR:
      return POKER_WINNING_CONFIDENCE_PAIR;
    default:
      return POKER_WINNING_CONFIDENCE_HIGH_CARD;
  }
}

void PokerWinning::fillRemaining() {
  uint8_t i, highest, current;
  struct Card highestCard(0x00);
  struct Card currentCard(0x00);

  // Set the kicker
  this->kicker = 0xFF;

  // Fill the remaining cards
  while(this->set.size() < POKER_WINNING_SET_SIZE) {
    highest = 0xFF;

    for(i = 0; i < this->full.size(); i++) {
      currentCard = this->full[i];
      if(Card::contains(&this->set, currentCard) != -1) continue;

      if(highest == 0xFF) {
        highestCard = currentCard;
        highest = highestCard.getValue();
      } else {
        current = currentCard.getValue();
        if(current != CARD_ACE && current < highest) continue;
        highestCard = currentCard;
        highest = current;
      }
    }

    if(highest == 0xFF) break;
    this->set.push_back(highestCard);
  }
  Card::sort(&this->set);
}